@@ -12,7 +12,7 @@ from .piezo import Piezo
 
 from .manager import Manager, open_manager, open_device
 from .interface import nodealiasproperty, nodealias, RpcError, Node, RpcNode, NodeAlias, NodeAliasProperty
-from .interface import RpcInterface, Interface
+from .interface import RpcInterface, Interface, RpcProperty, NodeProperty, InterfaceProperty
 from .interface import Int16, Int32, Int64, UInt16, UInt32, UInt64, Float, Double
 from .interface import INT  , DINT , LINT , UINT  , UDINT , ULINT  , REAL, LREAL
 from .tools import *
