@@ -79,6 +79,8 @@ def disconnect_ua_client(ua_client, obj):
     #     _ua_clients_references.pop(ua_client)
     
     
+def _ua_server_sid(client):
+    return client.server_url.netloc
     
 ####
 # Some dummy classes to work without any opc-ua server available
@@ -288,7 +290,10 @@ class NodeAlias:
             self.fget = fget
         if fset:
             self.fset = fset
-        
+    
+    def __repr__(self):
+        return "<{} key='{}'>".format(self.__class__.__name__, self._key)
+    
     @property
     def key(self):
         return self._key
@@ -303,6 +308,10 @@ class NodeAlias:
         s,_,p = self.key[::-1].partition(".")
         return s[::-1]
     
+    @property
+    def nodes(self):
+        return self._nodes
+        
     def get(self, data=None):
         """ get the node alias value from server or from data dictionary if given """
         if data is None:
@@ -328,7 +337,13 @@ class NodeAlias:
     def fset(*args):
         raise NotImplementedError("Node alias set function")
         
-class NodeAliasPropertyChildren(NodeAlias):
+class NodeAliasP(NodeAlias):
+    """ Same as :class:`NodeAlias` exept it brings an object with it
+    
+    The attached object will be sent to the fget and fset method as first argument. 
+    
+    :class:`NodeAliasP` is returned by a :class:`NodeAliaseProperty`
+    """
     def __init__(self, obj, key, nodes, fget=None, fset=None):
         self._obj = obj
         self._nodes = nodes
@@ -402,7 +417,7 @@ class NodeAliasProperty:
             node_alias = obj.__dict__[self.name]
         except KeyError:
             nodes = [obj.get_node(n) for n in self.node_names]
-            node_alias = NodeAliasPropertyChildren(obj, kjoin(obj.key,self.name), nodes, fget=self.fget, fset=self.fset)
+            node_alias = NodeAliasP(obj, kjoin(obj.key,self.name), nodes, fget=self.fget, fset=self.fset)
             # hopefully self.name shall be the same than the name in class definition 
             # otherwise side effects are expected 
             obj.__dict__[self.name] = node_alias
@@ -593,7 +608,26 @@ class Node:
         else:
             data[self] = value
     
+class NodeProperty:
+    def __init__(self, name, ua_name, vtype=None):
+        self.name = name
+        self.ua_name = ua_name
+        self.vtype = vtype
     
+    def __get__(self, parent, cls=None):
+        if parent is None:
+            return self
+        try:
+            node =  parent.__dict__[self.name]
+        except KeyError:    
+            sid = "ns={};s={}".format(parent._ua_namespace, kjoin(parent._ua_prefix, self.ua_name))
+            uaNode = parent._ua_client.get_node(sid)
+            node = parent.Node(kjoin(parent.key, self.name),uaNode, vtype=self.vtype, sid=_ua_server_sid(parent._ua_client))
+            parent.__dict__[self.name] = node
+        return node
+    
+    def __set__(self, val):
+        raise AttributeError('readonly attribute')          
     
 class RpcNode:
     """ Object representing an OPC-UA Rpc method 
@@ -682,6 +716,29 @@ class RpcNode:
         self.rcall(*args)
 
 
+
+class RpcProperty:
+    def __init__(self, name, ua_name, atypes=None):
+        self.name= name 
+        self.ua_name, self.ua_method = ksplit(ua_name)
+        self.atypes = atypes
+    
+    def __get__(self, parent, cls=None):
+        if parent is None:
+            return self
+        try:
+            node =  parent.__dict__[self.name]
+        except KeyError:    
+            sid = "ns={};s={}".format(parent._ua_namespace, kjoin(parent._ua_prefix, self.ua_name))
+            mid = "{}:{}".format(parent._ua_namespace, self.ua_method)
+            uaNode = parent._ua_client.get_node(sid)
+            node = parent.RpcNode(kjoin(parent.key, self.name), uaNode, mid,  atypes=self.atypes)
+            parent.__dict__[self.name] = node
+        return node
+    
+    def __set__(self, val):
+        raise AttributeError('readonly attribute')          
+
 def getkey(obj):
     return obj.key
 
@@ -695,12 +752,13 @@ class NodesWriter:
 def _dispatch_node_to_write(nodes_values):
     disptach = {} # None is for aliases 
     all_wv = {}
+    # Threat Aliases first and then the real client/server nodes
     for node, value in nodes_values.items():
-        if isinstance(node, NodeAlias):
+        if not isinstance(node, Node):
             node.set(nodes_values)
     
     for node, value in nodes_values.items():
-        if not isinstance(node, NodeAlias):
+        if isinstance(node, Node):
             _add_node_to_dispatch_writer(disptach,  all_wv, node, value)
     return disptach, all_wv
 
@@ -713,7 +771,7 @@ def _add_node_to_dispatch_writer(disptach, all_wv, node, value):
         params.uaclient = node._ua_node.server
         params.NodesToWrite = []
         disptach[node._sid] = params
-        
+    
     wv = ua.WriteValue()
     wv.NodeId = node._ua_node.nodeid
     wv.AttributeId = ua.AttributeIds.Value
@@ -759,9 +817,9 @@ def _dispatch_node_reader(nodes):
 
 def _add_node_to_dispatch_reader(disptach, aliases, node):
     
-    if isinstance(node, NodeAlias):
+    if not isinstance(node, Node):
         aliases.append( node )
-        for n in node._nodes:            
+        for n in getattr(node, "nodes", []):            
             _add_node_to_dispatch_reader(disptach,  aliases, n)
         return 
     
@@ -899,7 +957,8 @@ class Interface:
     
     
     Node = Node
-
+    __cashed_alias__ = False
+    
     def __init__(self, key, ua_client, map, ua_prefix, ua_namespace=DEFAULT_NAME_SPACE):
         self._ua_client = ua_client
         self._map = map
@@ -912,9 +971,12 @@ class Interface:
 
     def __getattr__(self, attr):
         try:
-            return self.__dict__[attr]
-        except KeyError:
-            return self.get_node(attr)
+            return object.__getattribute__(self, attr)
+        except AttributeError:
+            try:
+                return self.get_node(attr)
+            except KeyError:
+                raise AttributeError(attr)
 
     def __dir__(self):
         return ['get', 'set', 'gat_value_node'] + list(self._map.keys())
@@ -929,11 +991,26 @@ class Interface:
             except (KeyError,AttributeError):
                 continue
         return None
+    
         
+    @property
+    def all_native_nodes(self):
+        """ a list nodes of the interface """
+        return [self.get_node(k) for k in self.map.keys()]
+    
     @property
     def all_nodes(self):
         """ a list nodes of the interface """
-        return [self.get_node(k) for k in self.map.keys()]
+        _ = self.all_native_nodes # make sure all native nodes are cashed 
+        if not self.__cashed_alias__:
+            for sub in self.__class__.__mro__:
+                for k,v in sub.__dict__.items():
+                    if isinstance(v, (NodeAliasProperty)):
+                        getattr(self, k)
+            self.__cashed_alias__ = True
+        
+        return [v for v in self.__dict__.values() if isinstance(v, (Node,NodeAlias))]
+        #return [self.get_node(k) for k in self.map.keys()]
     
     @property
     def key(self):
@@ -969,18 +1046,20 @@ class Interface:
         for k,v in list(self.__dict__.items()):
             if isinstance(v, (Node, NodeAlias)):
                 self.__dict__.pop(k)
+        self.__cashed_alias__ = False
     
     def get_nodes(self, node_names=None):
         """ return a list of :class:`Node` from a list of names 
         
-        if node_names is None all the native nodes are return. The :class:`NodeAliasProperty` 
-        are not included.
+        if node_names is None all the nodes (inluding aliases) are return. 
+        If one want only the "native" nodes use the ``.all_native_nodes`` property
         
         
         !!! Warning if a given name startwith '_' this method may return something which is not 
-           a :class:`Node`. No check is done for performance raison.  
+           a :class:`Node` or :class:`NodeAlias`. No check is done for performance raison.  
         """
-        node_names = self.map.keys() if node_names is None else node_names
+        if node_names is None:
+            return self.all_nodes
         return [self.get_node(n) for n in node_names]
     
     def get_node(self, name):
@@ -1012,13 +1091,42 @@ class Interface:
                     raise KeyError('Unknown node %r'%name)
             
             else:
-                vNode = self.Node(kjoin(self.key, name), uaNode, vtype=self._get_vtype(name), sid=self._ua_client.server_url.netloc)
+                vNode = self.Node(kjoin(self.key, name), uaNode, vtype=self._get_vtype(name), sid=_ua_server_sid(self._ua_client))
                 # cash the object inside __dict__
                 self.__dict__[name] = vNode
                 return vNode
         
         return vNode
 
+
+class InterfaceProperty:
+    def __init__(self, map_key, cls=Interface):
+        self.map_key = map_key
+        self.cls = cls
+    
+    def finalize(self, device, interface):
+        pass
+    
+    def finalizer(self, func):
+        self.finalize = func
+        return self
+    
+    def __get__(self, device, cls=None):
+        if device is None:
+            return self
+        try:
+            interface = device.__dict__[self.map_key]
+        except KeyError:
+            if isinstance(self.cls, str):
+                cls = getattr(device, self.cls)
+            else:
+                cls = self.cls
+            
+            interface = cls(device.key, device._ua_client, device.map.get(self.map_key, {}), device.ua_prefix, ua_namespace=device.ua_namespace)
+            self.finalize(device, interface)
+            device.__dict__[self.map_key] = interface
+        return interface        
+
 class RpcInterface:
     """ Interface between opc-ua nodes of rpc method and pydevmgr set of :class:`RpcNodes` 
     
@@ -1062,10 +1170,14 @@ class RpcInterface:
 
     def __getattr__(self, attr):
         try:
-            return self.__dict__[attr]
-        except KeyError:
-            return self.get_method(attr)
-
+            return object.__getattribute__(self, attr)
+        except AttributeError:
+            try:
+                return self.get_method(attr)
+            except KeyError:
+                raise AttributeError(attr)
+    
+    
     @classmethod
     def _get_atypes(cl, key):
         """ look in class __annotations__ to find any parser for a given key """
