@@ -135,14 +135,19 @@ class ManagerStatInterface(Interface):
 class Manager:
     """ Manager object handling several devices 
     
-    Most likely the Manager will be initialized by :func:`Manager.from_config` or its alias :func:`open_manager`
+    .. note::
     
+        Most likely the Manager will be initialized by :func:`Manager.from_config` or its alias :func:`open_manager`
+    
+    If :func:`Manager.from_config` or :func:`open_manager` is used all the device prefixes will be
+    the key of the device manager.  
     
     Args:
         key (str): the key (prefix of all devices) of the manager
                 If None key is the 'server_id' defined inside the config dictionary
-        config (dic): The config dictionary (as red from yml config files by :func:`Manager.from_config`)
-        fits_prefix (str, optional): optional prefix for the Fits keyword (if ever used)
+        devices (dic): name/:class:`Device` pair dictionary 
+        fits_key (str, optional): the Fits keyword (if ever used) of the manager. 
+                   
     
     """
     # yes this dictionary is intended to be set on the class
@@ -151,46 +156,14 @@ class Manager:
     
     StatInterface = ManagerStatInterface    
     _stat = None
-
-    def __init__(self, key, config, fits_prefix='', config_file='', extra=None):
-        
-        server_id = config[K.SERVER_ID]
-        key = server_id if key is None else key
-        prefix, name = ksplit(key)
-        
-        server_config = config[server_id]
-        
-        device_names = server_config[K.DEVICES]
-        name = server_id if name is None else name
-        
-        fits_key = fjoin(fits_prefix, server_config.get(K.FITS_PREFIX, ''))  
-        
-        devices = {}
-        
+    
+    
+    def __init__(self, key, devices, fits_key='', cmdtout=60000, extra=None):
         self._key = key 
-        
         self._fits_key = fits_key
-        self._config_file = config_file # for future use 
-        
-        for device_name in device_names:
-            try:
-                dev_info = config[device_name]
-            except KeyError:
-                raise ValueError('device %r is not defined in the configuration file'%device_name)
-            dev_type = dev_info[K.TYPE]
-            cfg_gile = dev_info[K.CFGFILE]
-            try:
-                cl = self.get_device_type(dev_type)
-            except ValueError:
-                warning("The type %r is not implemented, %r will have a generic Device class"%(dev_type, device_name))
-                cl = self.Device
-                
-            devices[device_name] = cl.from_config(cfg_gile, device_name, key=kjoin(self.key, device_name), fits_prefix=self.fits_key)
-            
-        self._devices = devices
-        
+        self._devices = dict(devices)
         self._extra = {} if extra is None else extra
-
+        
     def __getattr__(self,attr):
         try:
             return self.__dict__[attr]
@@ -200,6 +173,13 @@ class Manager:
             except ValueError:
                 raise AttributeError("%r"%attr)
     
+    def __dir__(self):
+        lst = [d.name for d in self.devices()]
+        for sub in self.__class__.__mro__:
+            for k in sub.__dict__:
+                if not k.startswith('_'):
+                    lst.append(k)
+        return lst 
     
     @property
     def key(self):
@@ -242,7 +222,7 @@ class Manager:
     
     
     @classmethod
-    def from_config(cl, file_name, key=None, extra_file=None):
+    def from_config(cl, file_name, key=None, extra_file=None, fits_prefix=''):
         """ Create a :class:`Manager` object from its yaml configuration file 
         
         Args:
@@ -263,7 +243,31 @@ class Manager:
         else:
             extra = io.load_extra_of(file_name)
         
-        return cl(key, config, config_file=file_name, extra=extra)
+        server_id = config[K.SERVER_ID]
+        key = server_id if key is None else key
+        server_config = config[server_id]
+        
+        fits_key = fjoin(fits_prefix, server_config.get(K.FITS_PREFIX, ''))
+        device_names = server_config[K.DEVICES]
+        devices = {}
+        
+        for device_name in device_names:
+            try:
+                dev_info = config[device_name]
+            except KeyError:
+                raise ValueError('device %r is not defined in the configuration file'%device_name)
+            dev_type = dev_info[K.TYPE]
+            cfg_gile = dev_info[K.CFGFILE]
+            try:
+                Devcl = cl.get_device_type(dev_type)
+            except ValueError:
+                warning("The type %r is not implemented, %r will have a generic Device class"%(dev_type, device_name))
+                Devcl = cl.Device
+                
+            devices[device_name] = Devcl.from_config(cfg_gile, device_name, key=kjoin(key, device_name), fits_prefix=fits_key)
+            
+        return cl(key, devices, fits_key=fits_key, cmdtout=server_config.get('cmdtout',60000), extra=extra) 
+        #return cl(key, config, config_file=file_name, extra=extra)
         
     @classmethod
     def record_new_device_type(cl, type_name, dev_cls):
@@ -299,10 +303,12 @@ class Manager:
             return self._devices[name]
         except KeyError:
             raise ValueError('Unknown device %r'%name)
-
+    
+    @property
     def devices(self):
         """ return the list of children :class:`Device` like object """
-        return list(self._devices.values())
+        return DeviceIterator(self._devices)
+        #return list(self._devices.values())
 
     def device_names(self):
         """ return a list of child device names """
@@ -333,3 +339,21 @@ class Manager:
         """ Configure all child devices """
         for device in self.devices():
             device.configure()
+
+class DeviceIterator:
+    def __init__(self, devices):
+        self._devices = devices
+    
+    def __iter__(self):
+        return iter(self._devices.values())
+    
+    def __getitem__(self, item):
+        return self._devices[item]
+    
+    def __call__(self):
+        return list(self._devices.values())
+    
+    def names(self):
+        return list(self._devices)
+    
+    
\ No newline at end of file
