@@ -114,6 +114,10 @@ class ShutterStatInterface(Device.StatInterface):
     
     ERROR = ERROR
     SUBSTATE = SUBSTATE
+    
+    @nodealiasproperty("is_ready", ["substate"])
+    def is_ready(self, substate):
+        return substate in [self.SUBSTATE.NOT_OP_READY_OPEN, self.SUBSTATE.NOT_OP_READY_CLOSED]
         
     @nodealiasproperty("is_open", ["substate"])
     def is_open(self, substate):
@@ -162,26 +166,4 @@ class Shutter(Device):
     
 Manager.record_new_device_type('Shutter', Shutter)
 
-Manager.record_cfg_template('Shutter', """{name}:
-  type: Shutter
-  interface: Softing
-  identifier: PLC1                             # OPCUA Object Identifier
-  prefix: MAIN.Shutter1                       # OPCUA attribute prefix
-  simulated: false
-  ignored: false
-  address: {address}        # TO BE UPDATED  PLC address (OPCUA Server)
-  simaddr: opc.tcp://134.171.2.213:7576        # Simulation address
-  mapfile: "{cfg_dir}/mapShutter.yml"  # TO BE UPDATED 
-  fits_prefix: "SHUT1"
-  ctrl_config:
-    low_closed:     false
-    low_fault:      false                              # If T, signal is active low
-    low_open:       false                              # If T, signal is active low
-    low_switch:     false                              # If T, signal is active low
-    ignore_closed:  false                              # If T, ignore the signal
-    ignore_fault:   false                              # If T, ignore the signal
-    ignore_open:    false                              # If T, ignore the signal
-    initial_state:  false
-    timeout:       2000
-""")
 
