@@ -6,11 +6,13 @@ import time
 from datetime import datetime, timedelta
 import traceback
 import sys
+import itertools 
+from copy import deepcopy
 
 __all__ = ['wait', 'Waiter', 'get_utc', 'local_utc', 
            'local_time', 'FifoNodes', 'FifoOneNode', 
             'Downloader', 'Uploader', 'Player', 'Prefixed',
-            'suffix_setter', 'setitem', 'download', 'upload']
+            'suffix_setter', 'setitem', 'download', 'upload', 'Measurer']
 
 
 def get_local_time():
@@ -132,6 +134,29 @@ class FifoNodes(NodeAlias):
             self.fifo = deque([], maxlen)
         
 
+class Counter(NodeAlias):
+    """ A simple counter NodeAlis 
+    
+    Args:
+       key (str, optional): Node Alias key default is "counter"
+       start (int, optional): Start number (first returned number) default is 0
+       step (int, optional): counter increment step, default is 1
+    """
+    def __init__(self, key="counter", start=0, step=1):
+        self.key = key
+        self.counter = start        
+        self.step 
+    
+    def fget(self):
+        c = self.counter
+        self.counter += self.step
+        return c
+    
+    def reset(self, start=0):
+        """ reset the counter to start value (default = 0)"""
+        self.counter = start 
+        
+
 def setitem(d,k,v):
     d[k] = v
 
@@ -193,7 +218,7 @@ def upload(node_values):
 
 
 
-def wait(node_or_func_list, logic="all_true", period=0.1, timeout=60, data=None):
+def wait(node_or_func_list, logic="all_true", period=0.1, timeout=60, lag=0.0, data=None):
     """ wait until a list of function return True
 
     Args:
@@ -210,6 +235,16 @@ def wait(node_or_func_list, logic="all_true", period=0.1, timeout=60, data=None)
             the sleep time in second in between checks
         timeout (float, optional): timeout in second, a RuntimeError is raised if conditions
                          are still false after timeout
+        lag (float, optional): Add a flat time lag (in second) before starting to wait. 
+                    This could be used to make sure that the last operation has been digested by server. 
+                         
+                    Bellow the lag is used to make sure that when ``wait`` starts the motor is moving
+                          
+                    ::
+                             
+                            >>> mgr.motor1.move_rel(1.0, 0.25)
+                            >>> wait( [mgr.motor1.stat.is_standstill], lag=0.1 )
+        
         data (None, dict, optional):  If given input nodes values are taken from the 
                data dictionary which is expected to be updated in someother place.
 
@@ -228,7 +263,7 @@ def wait(node_or_func_list, logic="all_true", period=0.1, timeout=60, data=None)
             > wait( [is_arrived, camera_ready])
 
     """
-    Waiter(node_or_func_list, logic=logic, period=period, timeout=timeout, data=data).wait()
+    Waiter(node_or_func_list, logic=logic, period=period, timeout=timeout, data=data).wait(lag)
 
 def _all_true(functions, node_get):
     """ all_true(lst, nodes) -> return True if all function and node in list return True """
@@ -314,7 +349,7 @@ class Waiter:
         timeout (float): timeout in second, a RuntimeError is raised if conditions
                          are still false after timeout.
     """
-    def __init__(self, node_or_func_list, logic="all_true", period=0.1, timeout=60, data=None):        
+    def __init__(self, node_or_func_list, logic="all_true", period=0.1, timeout=60, data=None, stopper=lambda:None):        
         nodes = []
         functions = []
         for f in node_or_func_list:
@@ -344,18 +379,35 @@ class Waiter:
         self._node_get = node_get        
         self.period  = period
         self.timeout = timeout
-    
-    def wait(self):
-        """ run the wait condition """
+        self.stopper = stopper
         
-        check = self._check
-        node_get = self._node_get
+    def wait(self, lag=0.0):
+        """ run the wait condition 
+        
+        Args:
+            lag (float, optional): Add a flat time lag (in second) before starting to wait. 
+                        This could be used to make sure that the last operation has been digested by server. 
+                             
+                        Bellow the lag is used to make sure that when ``wait`` starts the motor is moving
+                              
+                        ::
+                                >>> waiter = Waiter([mgr.motor1.stat.is_standstill])
+                                >>> mgr.motor1.move_rel(1.0, 0.25)
+                                >>> waiter.wait( 0.1 )                
+        """
+        check     = self._check
+        node_get  = self._node_get
         functions = self._functions 
-        timeout = self.timeout
-        period = self.period
-                
-        s_time = time.time()    
+        timeout   = self.timeout
+        period    = self.period
+        stopper   = self.stopper 
+        s_time = time.time()
+        if lag>0.0:
+            time.sleep(lag)
+            
         while not check(functions, node_get):
+            stopper() # raise a SopIteration if needed to stop. The StopIteration shall be catched 
+                      # at higher level like in Measure.run or Download.run  
             if (time.time()-s_time)>timeout:
                 raise RuntimeError('wait timeout')
             time.sleep(period)
@@ -690,11 +742,14 @@ class Downloader:
             stopsignal (callable, optional): a function returning True to stop the loop or False to continue
             
         """
-        while not stopsignal():
-            s_time = time.time()
-            self.download()
-            sleepfunc( max( period-(time.time()-s_time), 0))
-    
+        try:
+            while not stopsignal():
+                s_time = time.time()
+                self.download()
+                sleepfunc( max( period-(time.time()-s_time), 0))
+        except StopIteration: # any downloader call back can send a StopIteration to stop the runner 
+            return 
+            
     def runner(self, period=1.0, stopsignal=lambda : False, sleepfunc=time.sleep): 
         """ Create a function to run the download in a loop 
         
@@ -834,6 +889,11 @@ class Prefixed:
         
         Shall be avoided to use in a :class:`Prefixed` object
         """
+        if not self._prefix : 
+            for k in self._data.keys():
+                yield k 
+            return 
+        
         pref = self._prefix+"."
         lp = len(pref)
         for k in self._data.keys():
@@ -845,6 +905,11 @@ class Prefixed:
         
         Shall be avoided to use in a :class:`Prefixed` object
         """
+        if not self._prefix : 
+            for k,v in self._data.items():
+                yield k , v    
+            return 
+            
         pref = self._prefix+"."
         lp = len(pref)
         for k, v in self._data.items():
@@ -856,6 +921,11 @@ class Prefixed:
         
         Shall be avoided to use in a :class:`Prefixed` object
         """
+        if not self._prefix : 
+            for v in self._data.values():
+                yield v    
+            return 
+                
         pref = self._prefix+"."
         for k, v in self._data.items():
             if isinstance(k, str) and k.startswith(pref):
@@ -866,10 +936,133 @@ class Prefixed:
         
         Shall be avoided to use in a :class:`Prefixed` object
         """
+        if not self._prefix:
+            self._data.clear()
+            return 
+        
         pref = self._prefix+"."
         for k, v in list(self._data.items()):
             if isinstance(k, str) and k.startswith(pref):
                 self._data.pop(k)
     
+class Measurer:
+    """ A base class tool to take sequence of measurements 
+    
+    The class does not do that much but offer a structure for measurements  
     
     
+    
+    Args:
+        nodes (iterable): a list of nodes          
+        measurements (list, optional):  This is the output list of measurements. 
+                !! warning the list is cleared int the default init function
+        data (dict, optional): If given the measurement are taken from this data. This is only used when 
+                data is fed in an other process. 
+                
+                
+                
+    Exemple:
+       
+       Take measurement of motor position after a sequence of movement. 
+       
+       class MeasureMotorPos(Measurer):
+            def __init__(self, motor, file):
+                self.motor = motor
+                self.file = file
+                self.f = None        
+                self.counter = 0 
+                
+                super(Measurer, self).__init__( [motor.stat.pos_actual, motor.stat.pos_error] )
+            
+            def init(self): 
+                self.measurements.clear()           
+                self.counter = 0    
+                self.f = open(self.file)
+                self.f.write( "#pos_target velocity pos pos_error image\n")
+               
+            def before(self, pos_vel):
+               self.motor.move_rel(*pos_vel)
+               wait( [self.motor.stat.is_standstill], lag=0.1 )
+              
+            def after(self, pos_vel, data):
+                # Do something, for instance take an image from a camera
+                time.sleep(0.2)
+                self.counter += 1 
+                img_file = "image_%03d.tiff"%self.counter
+                # image = take_image_of_my_lab_cam()
+                # save_image(image, img_file)
+                data['image'] = img_file
+            
+            def end(self):
+                pass
+                     
+                
+               
+               
+    
+    """
+    def __init__(self, nodes, measurements=None,  data=None):    
+        
+        self.last_data  = {}
+        self.measurements = [] if measurements is None else measurements        
+        self.data = data
+        self.nodes = nodes 
+        self.aborded = False
+    
+    def stopper(self):
+        """ A stop function which may raise a SoptIteration to stop the loop """
+        pass
+    
+    def before(self, arg):
+        """ Action to be executed before measurements. Must be defined by user """
+        pass
+    
+    def after(self, arg, data):
+        """ Action to be executed after measurements. Must be defined by user """
+        pass
+        
+    def init(self):
+        """ called when a sequence of measurements is started. Must be defined by user 
+        
+        Buy default this just clear the measurements list 
+        """
+        self.measurements.clear()
+        
+    def end(self):
+        """ end a sequence of measurements, must be defined by user """
+        pass
+    
+    def measure(self, arg):
+        
+        self.before(arg)
+        
+        last_data = {}
+        
+        if self.data:
+            last_data = {n.key:self.data[n.key] for n in self.nodes}
+        else:
+            last_data = {}
+            download(self.nodes, last_data)
+            
+        self.after(arg, last_data)
+        self.measurements.append(last_data)    
+        return last_data
+    
+    def run(self, args, delay=0.01):
+        self.init()
+        self.aborded = False
+        try:
+            for arg in args:
+                self.measure(arg)
+                time.sleep(delay)
+        except StopIteration:
+            self.aborded = True
+            self.end()
+        else:
+            self.end() 
+        return self.measurements
+    
+                
+            
+        
+        
